/**************************************************************************
 *
 * STATUS.C -  NetSaint Status CGI
 *
 * Written by Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified: 08-28-1999
 *
 * License:
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *************************************************************************/

#include "../common/config.h"
#include "../common/locations.h"
#include "cgiutils.h"
#include "getcgi.h"
#include "../common/common.h"

#define MAX_MESSAGE_BUFFER		4096

#define DISPLAY_HOSTS			0
#define DISPLAY_HOSTGROUPS		1

#define STYLE_OVERVIEW			0
#define STYLE_DETAIL			1

void show_host_status_totals(void);
void show_service_status_totals(void);
void show_service_detail(void);
void show_hostgroup_detail(void);
void show_hostgroup_overviews(void);
void show_hostgroup_overview(hostgroup *);
void show_hostgroup_member_overview(hoststatus *,int);
void show_hostgroup_member_service_status_totals(char *);

void grab_host_alerts(void);
void alert_window(char *);

void document_header();
void document_footer();
int process_cgivars();

extern char main_config_file[MAX_FILENAME_LENGTH];
extern char hosts_config_file[MAX_FILENAME_LENGTH];
extern char url_html_path[MAX_FILENAME_LENGTH];
extern char url_docs_path[MAX_FILENAME_LENGTH];
extern char url_images_path[MAX_FILENAME_LENGTH];
extern char url_logo_images_path[MAX_FILENAME_LENGTH];
extern char url_cgibin_path[MAX_FILENAME_LENGTH];
extern char log_file[MAX_FILENAME_LENGTH];
extern char status_file[MAX_FILENAME_LENGTH];

extern hostgroup *hostgroup_list;
extern hoststatus *hoststatus_list;
extern servicestatus *servicestatus_list;

char alert_message[MAX_MESSAGE_BUFFER];
char host_name[MAX_HOSTNAME_LENGTH];
char hostgroup_name[MAX_HOSTGROUPNAME_LENGTH];
int host_alert=FALSE;
int show_all_hosts=FALSE;
int show_all_hostgroups=FALSE;
int display_type=DISPLAY_HOSTS;
int overview_columns=3;
int display_popup=TRUE;
int hostgroup_style_type=STYLE_OVERVIEW;

int main(void){
	int result=OK;
	int error=FALSE;
	time_t current_time;
	char date_time[48];
	
	error=process_cgivars();

	document_header();


	if(error==TRUE){
		printf("<p><div align=\"center\"><font color=\"red\"><strong>");
		printf("Invalid argument supplied");
		printf("</strong></font></div></p>\n");
	        }

	else{
	
		reset_cgi_vars();

		result=read_cgi_config_file(DEFAULT_CGI_CONFIG_FILE);
		if(result==ERROR){
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open CGI configuration file '%s' for reading!\n",DEFAULT_CGI_CONFIG_FILE);
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		result=read_main_config_file(main_config_file);
		if(result==ERROR){
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open main configuration file '%s' for reading!\n",main_config_file);
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		result=read_status_log(status_file);
		if(result==ERROR){
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open status log '%s' for reading!\n",status_file);
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		result=read_hosts_config_file(hosts_config_file,READ_HOSTGROUPS);
		if(result==ERROR){
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open host configuration file '%s' for reading!\n",hosts_config_file);
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
                        }

		/* show the JavaScript alert window if a host is down or unreachable */
		grab_host_alerts();
		if(host_alert==TRUE)
			alert_window(alert_message);
	        }


	/* begin top table */
	printf("<p>\n");
	printf("<table border=0 width=100%%>\n");
	printf("<tr>\n");
	printf("<td align=left valign=top>\n");

	/* left column of the first row */
	printf("<table border=1>\n");
	printf("<tr><td bgcolor=%s>\n",HTML_STATUS_BG_EVEN_COLOR);
	printf("<font size=+2><b>Current Network Status</b></font><br>\n");
	printf("<font size=-2>\n");
	time(&current_time);
	get_time_string(&current_time,date_time,sizeof(date_time));
	printf("Last Updated: %s<BR>",date_time);
	printf("Updated every %s seconds<br>\n",HTML_REFRESH_RATE);
	printf("NetSaint Network Monitor - <a href=\"http://www.netsaint.org\" target=\"_new\">www.netsaint.org</a><br>\n");
	printf("</font>\n");
	printf("</td></tr>\n");

	if(display_type==DISPLAY_HOSTS){

		printf("<tr><td bgcolor=%s>\n",HTML_STATUS_BG_ODD_COLOR);
		printf("<font size=-2>\n");
		printf("<a href=\"history.cgi?host=%s\">View history for %s</a><br>\n",(show_all_hosts==TRUE)?"all":host_name,(show_all_hosts==TRUE)?"all hosts":"this host");
		printf("<a href=\"notifications.cgi?host=%s\">View notifications for %s</a>\n",(show_all_hosts==TRUE)?"all":host_name,(show_all_hosts==TRUE)?"all hosts":"this host");
		printf("</font>\n");
		printf("</td></tr>\n");

		if(show_all_hosts==FALSE){
			printf("<tr><td bgcolor=%s>\n",HTML_STATUS_BG_EVEN_COLOR);
			printf("<font size=-2>\n");
			printf("<a href=\"status.cgi?host=all%s\">View status detail for all hosts</a>\n",(display_popup==TRUE)?"":"&nopopup");
			printf("</font>\n");
			printf("</td></tr>\n");
	                }

	        }

	else{

		if(show_all_hostgroups==FALSE){

			if(hostgroup_style_type==STYLE_OVERVIEW){
				printf("<tr><td bgcolor=%s>\n",HTML_STATUS_BG_ODD_COLOR);
				printf("<font size=-2>\n");
				printf("<a href=\"status.cgi?hostgroup=all%s\">View status overview for all host groups</a>\n",(display_popup==TRUE)?"":"&nopopup");
				printf("</font>\n");
				printf("</td></tr>\n");
				printf("<tr><td bgcolor=%s>\n",HTML_STATUS_BG_EVEN_COLOR);
				printf("<font size=-2>\n");
				printf("<a href=\"status.cgi?hostgroup=%s&style=detail%s\">View status detail for this host group</a>\n",hostgroup_name,(display_popup==TRUE)?"":"&nopopup");
				printf("</font>\n");
				printf("</td></tr>\n");
			        }
			else{
				printf("<tr><td bgcolor=%s>\n",HTML_STATUS_BG_ODD_COLOR);
				printf("<font size=-2>\n");
				printf("<a href=\"status.cgi?hostgroup=%s%s\">View status overview for this host group</a>\n",hostgroup_name,(display_popup==TRUE)?"":"&nopopup");
				printf("</font>\n");
				printf("</td></tr>\n");
				printf("<tr><td bgcolor=%s>\n",HTML_STATUS_BG_EVEN_COLOR);
				printf("<font size=-2>\n");
				printf("<a href=\"status.cgi?hostgroup=all&style=detail%s\">View status detail for all host groups</a>\n",(display_popup==TRUE)?"":"&nopopup");
				printf("</font>\n");
				printf("</td></tr>\n");
			        }
		        }
		else{

			if(hostgroup_style_type==STYLE_OVERVIEW){
				printf("<tr><td bgcolor=%s>\n",HTML_STATUS_BG_ODD_COLOR);
				printf("<font size=-2>\n");
				printf("<a href=\"status.cgi?hostgroup=all&style=detail%s\">View status detail for all host groups</a>\n",(display_popup==TRUE)?"":"&nopopup");
				printf("</font>\n");
				printf("</td></tr>\n");
			        }
			else{
				printf("<tr><td bgcolor=%s>\n",HTML_STATUS_BG_ODD_COLOR);
				printf("<font size=-2>\n");
				printf("<a href=\"status.cgi?hostgroup=all%s\">View status overview for all host groups</a>\n",(display_popup==TRUE)?"":"&nopopup");
				printf("</font>\n");
				printf("</td></tr>\n");
			        }
		        }
	        }

	printf("</table>\n");
	printf("</td>\n");

	/* middle column of top row */
	printf("<td align=center valign=bottom>\n");
	show_host_status_totals();
	printf("</td>\n");


	/* right hand column of top row */
	printf("<td align=right valign=bottom>\n");
	show_service_status_totals();
	printf("</td>\n");

	/* end of top table */
	printf("</tr>\n");
	printf("</table>\n");
	printf("</p>\n");

	/* bottom portion of screen - service or hostgroup detail */
	if(display_type==DISPLAY_HOSTS)
		show_service_detail();
	else{
		if(hostgroup_style_type==STYLE_OVERVIEW)
			show_hostgroup_overviews();
		else
			show_service_detail();
	        }

	document_footer();

	free_memory();

	return OK;
        }


void grab_host_alerts(void){
	char message_buffer[MAX_INPUT_BUFFER];
	hoststatus *temp_hoststatus=NULL;
	

	for(temp_hoststatus=hoststatus_list;temp_hoststatus!=NULL;temp_hoststatus=temp_hoststatus->next)
		if(temp_hoststatus->status!=HOST_UP && temp_hoststatus->status!=HOST_PENDING){

			if(host_alert==FALSE){
				host_alert=TRUE;
				strcpy(alert_message,"");
			        }

			snprintf(message_buffer,sizeof(message_buffer)-1,"<tr><td valign='center'><IMG SRC='%s%s' BORDER=0 ALIGN='CENTER'></td><td>Host <b>%s</b> is <b>%s</b>!</td></tr>",url_images_path,HOST_CRITICAL_ICON,temp_hoststatus->host_name,(temp_hoststatus->status==HOST_DOWN)?"down":"unreachable");
			strncat(alert_message,message_buffer,sizeof(alert_message)-strlen(alert_message));
		        }

	return;
        }


void alert_window(char *alert_message){

	if(display_popup==TRUE){

		printf("<script language=\"javascript\">\n");
		printf("<!-- Hide code from old browsers\n");

		printf("var wnd;\n");
		printf("wnd = window.open(\"\",\"alert\",\"width=400,height=200,scrollbars=yes,statusbar=no,toolbar=no\");\n");
		printf("wnd.document.write(\"<title>NetSaint Host Alert Console</title>\");\n");
		printf("wnd.document.write(\"<body bgcolor='#EEEEF4' text='#000000'>\");\n");

		/* want to play a sound when the alert window pops up?  Alter the <EMBED> tag to include some kind of audio file.
		   Note that your browser must recognize the sound file format and will have to know what to do with it  */
		/*printf("wnd.document.write(\"<embed src='somesound.wav'>\");\n");*/

		printf("wnd.document.write(\"<p>\");\n");
		printf("wnd.document.write(\"<center><font size=-2>");
		printf("<a href='' onclick='self.close()'>Close This Window</a>");
		printf("&nbsp;&nbsp;|&nbsp;&nbsp;");
		printf("<a href='");
		printf("status.cgi?");
		if(display_type==DISPLAY_HOSTS){
			printf("host=%s",host_name);
		        }
		else{
			printf("hostgroup=%s",hostgroup_name);
			if(hostgroup_style_type==STYLE_DETAIL)
				printf("&style=detail");
			else
				printf("&columns=%d",overview_columns);
		        }
		printf("&nopopup");
		printf("' target='main'>Disable This Window</a>");
		printf("</font></center>\");\n");
		printf("wnd.document.write(\"</p>\");\n");
		printf("wnd.document.write(\"<p>\");\n");
		printf("wnd.document.write(\"<table border='0'>\");\n");
		printf("wnd.document.write(\"%s\");\n",alert_message);
		printf("wnd.document.write(\"</table>\");\n");
		printf("wnd.document.write(\"</p>\");\n");
		printf("wnd.document.write(\"</body>\");\n");
		printf("//-->\n");
		printf("</script>\n");
	        }
        }


void document_header(void){

	printf("Content-type: text/html\r\n\r\n");

	printf("<html>\n");
	printf("<head>\n");
	printf("<META HTTP-EQUIV=\"Refresh\" CONTENT=\"%s\">\n",HTML_REFRESH_RATE);
	printf("<META HTTP-EQUIV=\"Pragma\" CONTENT=\"no-cache\">\n");
	printf("<title>\n");
	printf("Current Service Status\n");
	printf("</title>\n");
	printf("</head>\n");
	printf("<body onunload=\"if (window.wnd && wnd.close) wnd.close(); return true;\">\n");

	return;
        }


void document_footer(void){

	printf("</body>\n");
	printf("</html>\n");

	return;
        }


int process_cgivars(void){
	char **variables;
	int error=FALSE;
	int x;

	variables=getcgivars();

	for(x=0;variables[x]!=NULL;x++){

		/* do some basic length checking on the variable identifier to prevent buffer overflows */
		if(strlen(variables[x])>=MAX_INPUT_BUFFER-1){
			x++;
			continue;
		        }

		/* we found the hostgroup argument */
		else if(!strcmp(variables[x],"hostgroup")){
			display_type=DISPLAY_HOSTGROUPS;
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* protect against buffer overflows */
			if(strlen(variables[x])>=MAX_HOSTGROUPNAME_LENGTH-1){
				strcpy(hostgroup_name,"all");
			        }
			else
				strcpy(hostgroup_name,variables[x]);

			if(!strcmp(hostgroup_name,"all"))
				show_all_hostgroups=TRUE;
			else
				show_all_hostgroups=FALSE;
		        }

		/* we found the host argument */
		else if(!strcmp(variables[x],"host")){
			display_type=DISPLAY_HOSTS;
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* protect against buffer overflows */
			if(strlen(variables[x])>=MAX_HOSTNAME_LENGTH-1){
				strcpy(host_name,"all");
			        }
			else
				strcpy(host_name,variables[x]);

			if(!strcmp(host_name,"all"))
				show_all_hosts=TRUE;
			else
				show_all_hosts=FALSE;
		        }

		/* we found the columns argument */
		else if(!strcmp(variables[x],"columns")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			overview_columns=atoi(variables[x]);
			if(overview_columns<=0)
				overview_columns=1;
		        }

		/* we found the popup window argument */
		else if(!strcmp(variables[x],"nopopup")){
			display_popup=FALSE;
		        }

		/* we found the hostgroup style argument */
		else if(!strcmp(variables[x],"style")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			if(!strcmp(variables[x],"detail"))
				hostgroup_style_type=STYLE_DETAIL;
		        }
	        }

	return error;
        }



void show_service_status_totals(void){
	int total_ok=0;
	int total_warning=0;
	int total_unknown=0;
	int total_critical=0;
	int total_pending=0;
	servicestatus *temp_servicestatus;
	int count_service;


	for(temp_servicestatus=servicestatus_list;temp_servicestatus!=NULL;temp_servicestatus=temp_servicestatus->next){

		count_service=0;
		if(display_type==DISPLAY_HOSTS && (show_all_hosts==TRUE || !strcmp(host_name,temp_servicestatus->host_name)))
			count_service=1;
		else if(display_type==DISPLAY_HOSTGROUPS && (show_all_hostgroups==TRUE || is_member_of_hostgroup(find_hostgroup(hostgroup_name),temp_servicestatus->host_name)))
			count_service=1;

		if(count_service){

			if(temp_servicestatus->status==SERVICE_CRITICAL)
				total_critical++;
			else if(temp_servicestatus->status==SERVICE_HOST_DOWN)
				total_critical++;
			else if(temp_servicestatus->status==SERVICE_UNREACHABLE)
				total_critical++;
			else if(temp_servicestatus->status==SERVICE_WARNING)
				total_warning++;
			else if(temp_servicestatus->status==SERVICE_UNKNOWN)
				total_unknown++;
			else if(temp_servicestatus->status==SERVICE_RECOVERY)
				total_ok++;
			else if(temp_servicestatus->status==SERVICE_OK)
				total_ok++;
			else if(temp_servicestatus->status==SERVICE_PENDING)
				total_pending++;
			else
				total_ok++;
		        }
	        }


	printf("<DIV ALIGN=\"CENTER\">\n");
	printf("<B>Service Status Totals</B>\n");
	printf("<TABLE BORDER=\"1\" bgcolor=\"#EEEEEE\">\n");
	printf("<TR>\n");
	printf("<TD bgcolor=\"%s\"><FONT color=\"%s\"><B>Ok</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Warning</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Unknown</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("<TD bgcolor=\"%s\"><FONT color=\"%s\"><B>Critical</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("<TD bgcolor=\"%s\"><FONT color=\"%s\"><B>Pending</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("</TR>\n");

	printf("<TR>\n");
	if(total_ok>0)
	        printf("<TD align=\"center\" bgcolor=\"%s\">%d</TD>\n",OK_COLOR,total_ok);
	else
	        printf("<TD align=\"center\">%d</TD>\n",total_ok);
	if(total_warning>0)
	        printf("<TD align=\"center\" bgcolor=\"%s\"><BLINK>%d</BLINK></TD>\n",WARNING_COLOR,total_warning);
	else
	        printf("<TD align=\"center\">%d</TD>\n",total_warning);
	if(total_unknown>0)
	        printf("<TD align=\"center\" bgcolor=\"%s\"><BLINK>%d</BLINK></TD>\n",UNKNOWN_COLOR,total_unknown);
	else
	        printf("<TD align=\"center\">%d</TD>\n",total_unknown);
	if(total_critical>0)
	        printf("<TD align=\"center\" bgcolor=\"%s\"><BLINK>%d</BLINK></TD>\n",CRITICAL_COLOR,total_critical);
	else
	        printf("<TD align=\"center\">%d</TD>\n",total_critical);
	if(total_pending>0)
	        printf("<TD align=\"center\" bgcolor=\"%s\">%d</TD>\n",PENDING_COLOR,total_pending);
	else
	        printf("<TD align=\"center\">%d</TD>\n",total_pending);
	printf("</TR>\n");
	printf("</TABLE>\n");
	printf("</DIV>\n");

	return;
        }


void show_host_status_totals(void){
	int total_up=0;
	int total_down=0;
	int total_unreachable=0;
	int total_pending=0;
	hoststatus *temp_hoststatus;
	int count_host;


	for(temp_hoststatus=hoststatus_list;temp_hoststatus!=NULL;temp_hoststatus=temp_hoststatus->next){

		count_host=0;
		if(display_type==DISPLAY_HOSTS && (show_all_hosts==TRUE || !strcmp(host_name,temp_hoststatus->host_name)))
			count_host=1;
		else if(display_type==DISPLAY_HOSTGROUPS && (show_all_hostgroups==TRUE || is_member_of_hostgroup(find_hostgroup(hostgroup_name),temp_hoststatus->host_name)))
			count_host=1;

		if(count_host){

			if(temp_hoststatus->status==HOST_UP)
				total_up++;
			else if(temp_hoststatus->status==HOST_DOWN)
				total_down++;
			else if(temp_hoststatus->status==HOST_UNREACHABLE)
				total_unreachable++;
			else if(temp_hoststatus->status==HOST_PENDING)
				total_pending++;
			else
				total_up++;
		        }
	        }


	printf("<DIV ALIGN=\"CENTER\">\n");
	if(display_type==DISPLAY_HOSTGROUPS || (display_type==DISPLAY_HOSTS && show_all_hosts==TRUE))
		printf("<B>Host Status Totals</B>\n");
	else
		printf("<B>Host '%s' Status</B>\n",host_name);
	printf("<TABLE BORDER=\"1\" bgcolor=\"#EEEEEE\">\n");
	printf("<TR>\n");

	printf("<TD bgcolor=\"%s\"><FONT color=\"%s\"><B>Up</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Down</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Unreachable</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("<TD bgcolor=\"%s\"><FONT color=\"%s\"><B>Pending</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("</TR>\n");

	printf("<TR>\n");
	if(total_up>0)
	        printf("<TD align=\"center\" bgcolor=\"%s\">%d</TD>\n",HOST_UP_COLOR,total_up);
	else
	        printf("<TD align=\"center\">%d</TD>\n",total_up);
	if(total_down>0)
	        printf("<TD align=\"center\" bgcolor=\"%s\"><BLINK>%d</BLINK></TD>\n",HOST_DOWN_COLOR,total_down);
	else
	        printf("<TD align=\"center\">%d</TD>\n",total_down);
	if(total_unreachable>0)
	        printf("<TD align=\"center\" bgcolor=\"%s\"><BLINK>%d</BLINK></TD>\n",HOST_UNREACHABLE_COLOR,total_unreachable);
	else
	        printf("<TD align=\"center\">%d</TD>\n",total_unreachable);
	if(total_pending>0)
	        printf("<TD align=\"center\" bgcolor=\"%s\">%d</TD>\n",HOST_PENDING_COLOR,total_pending);
	else
	        printf("<TD align=\"center\">%d</TD>\n",total_pending);
	printf("</TR>\n");
	printf("</TABLE>\n");
	printf("</DIV>\n");

	return;
        }


void show_service_detail(void){
	time_t t;
	char date_time[48];
	char status[MAX_INPUT_BUFFER];
	char status_color[MAX_INPUT_BUFFER];
	char status_bg_color[MAX_INPUT_BUFFER];
	char last_host[MAX_INPUT_BUFFER]="";
	int new_host=FALSE;
	servicestatus *temp_status;
	hostgroup *temp_hostgroup;
	int odd=0;


	printf("<P>\n");
	printf("<DIV ALIGN=CENTER>\n");
	if(display_type==DISPLAY_HOSTS){
		if(show_all_hosts==TRUE)
			printf("<B>Service Status Details For All Hosts</B>\n");
		else
			printf("<B>Service Status Details For Host '%s'</B>\n",host_name);
	        }
	else{
		if(show_all_hostgroups==TRUE)
			printf("<B>Service Status Details For All Host Groups</B>\n");
		else
			printf("<B>Service Status Details For Host Group '%s'</B>\n",hostgroup_name);
	        }
	printf("<TABLE BORDER=0 width=100%%>\n");
	printf("<TR>\n");
	printf("<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Host</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Service</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Status</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Last Updated</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Attempt</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Service Information</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("</TR>\n");

	for(temp_status=servicestatus_list;temp_status!=NULL;temp_status=temp_status->next){

		if(display_type==DISPLAY_HOSTGROUPS){

			/* see if this host is a member of the hostgroup */
			if(show_all_hostgroups==FALSE){
				temp_hostgroup=find_hostgroup(hostgroup_name);
				if(temp_hostgroup==NULL)
					continue;
				if(is_member_of_hostgroup(temp_hostgroup,temp_status->host_name)==FALSE)
					continue;
		                }
		        }

		/* get the last service check time */
		t=temp_status->last_check;
		get_time_string(&t,date_time,sizeof(date_time));
		if(temp_status->last_check==0L)
			strcpy(date_time,"N/A");


		if(temp_status->status==SERVICE_PENDING){
			strcpy(status,"PENDING");
			strcpy(status_color,PENDING_COLOR);
		        }
		else if(temp_status->status==SERVICE_OK){
			strcpy(status,"OK");
			strcpy(status_color,OK_COLOR);
		        }
		else if(temp_status->status==SERVICE_RECOVERY){
			strcpy(status,"RECOVERY");
			strcpy(status_color,RECOVERY_COLOR);
		        }
		else if(temp_status->status==SERVICE_WARNING){
			strcpy(status,"WARNING");
			strcpy(status_color,WARNING_COLOR);
			strcpy(status_bg_color,WARNING_BG_COLOR);
		        }
		else if(temp_status->status==SERVICE_UNKNOWN){
			strcpy(status,"UNKNOWN");
			strcpy(status_color,UNKNOWN_COLOR);
			strcpy(status_bg_color,UNKNOWN_BG_COLOR);
		        }
		else if(temp_status->status==SERVICE_CRITICAL){
			strcpy(status,"CRITICAL");
			strcpy(status_color,CRITICAL_COLOR);
			strcpy(status_bg_color,CRITICAL_BG_COLOR);
		        }
		else if(temp_status->status==SERVICE_HOST_DOWN){
			strcpy(status,"HOST DOWN");
			strcpy(status_color,CRITICAL_COLOR);
			strcpy(status_bg_color,CRITICAL_BG_COLOR);
		        }
		else if(temp_status->status==SERVICE_UNREACHABLE){
			strcpy(status,"UNREACHABLE");
			strcpy(status_color,CRITICAL_COLOR);
			strcpy(status_bg_color,CRITICAL_BG_COLOR);
		        }


		if((display_type==DISPLAY_HOSTS && (show_all_hosts==TRUE || !strcmp(host_name,temp_status->host_name))) || display_type==DISPLAY_HOSTGROUPS ){

			if(strcmp(last_host,temp_status->host_name))
				new_host=TRUE;
			else
				new_host=FALSE;

			if(new_host==TRUE){
				if(strcmp(last_host,"")){
					printf("<TR><TD colspan=6><FONT SIZE=-2></FONT></TD></TR>\n");
					printf("<TR><TD colspan=6><FONT SIZE=-2></FONT></TD></TR>\n");
				        }
				else{
				        }
			        }

			if(odd)
				odd=0;
			else
				odd=1;

			if(temp_status->status==SERVICE_OK || temp_status->status==SERVICE_PENDING)
				printf("<TR bgcolor=%s>\n",(odd)?HTML_STATUS_BG_ODD_COLOR:HTML_STATUS_BG_EVEN_COLOR);
			else
				printf("<TR bgcolor=%s>\n",status_bg_color);

			printf("<TD>");
			if(new_host==TRUE){
				printf("<TABLE BORDER=0 WIDTH=\"100%%\" cellpadding=0 cellspacing=0>\n");
				printf("<TR>\n");
				printf("<TD align=left valign=center><A HREF=\"%shosts.html#%s\"><FONT SIZE=-1>%s</FONT></A></TD>\n",url_html_path,temp_status->host_name,temp_status->host_name);
				/*printf("<TD align=right valign=center><IMG SRC=\"%s%s\" BORDER=0 WIDTH=15 HEIGHT=15 ALT=\"Something...\"></TD>\n",url_logo_images_path,"unknown.png");*/
				printf("</TR>\n");
				printf("</TABLE>\n");
			        }
			printf("</TD>\n");

			printf("<TD><A HREF=\"%sservices.html#%s;%s\"><FONT SIZE=-1>%s</FONT></A></TD>\n",url_html_path,temp_status->host_name,temp_status->description,temp_status->description);	
			printf("<TD bgcolor=\"%s\"><A HREF=\"%sstatuslevels.html#%s\"><FONT SIZE=-1>%s</FONT></A></TD>\n",status_color,url_docs_path,status,status);
			printf("<TD><FONT SIZE=-1>%s</FONT></TD>\n",date_time);
			printf("<TD><FONT SIZE=-1>%d/%d</FONT></TD>\n",temp_status->current_attempt,temp_status->max_attempts);
			printf("<TD><FONT SIZE=-1>%s</FONT></TD>\n",temp_status->information);
			printf("</TR>\n");

			strcpy(last_host,temp_status->host_name);
		        }

	       }

	printf("</TABLE>\n");
	printf("</DIV>\n");
	printf("</P>\n");

	return;
        }


void show_hostgroup_detail(void){
	time_t t;
	char date_time[48];
	char status[MAX_INPUT_BUFFER];
	char status_color[MAX_INPUT_BUFFER];
	char status_bg_color[MAX_INPUT_BUFFER];
	servicestatus *temp_status;
	hostgroup *temp_hostgroup;
	int odd=0;


	printf("<P>\n");
	printf("<DIV ALIGN=CENTER>\n");
	if(show_all_hostgroups==TRUE)
		printf("<B>Service Status Details For All Host Groups</B>\n");
	else
		printf("<B>Service Status Details For Host Group '%s'</B>\n",hostgroup_name);
	printf("<TABLE BORDER=0 width=100%%>\n");
	printf("<TR>\n");
	printf("<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Host</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Service</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Status</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Last Updated</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Attempt</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Service Information</B></FONT></TD>\n",HTML_STATUS_BG_COLOR,HTML_STATUS_FG_COLOR);
	printf("</TR>\n");


	for(temp_status=servicestatus_list;temp_status!=NULL;temp_status=temp_status->next){

		/* see if this host is a member of the hostgroup */
		if(show_all_hostgroups==FALSE){
			temp_hostgroup=find_hostgroup(hostgroup_name);
			if(temp_hostgroup==NULL)
				continue;
			if(is_member_of_hostgroup(temp_hostgroup,temp_status->host_name)==FALSE)
				continue;
		        }

		/* get the last service check time */
		t=temp_status->last_check;
		get_time_string(&t,date_time,sizeof(date_time));
		if(temp_status->last_check==0L)
			strcpy(date_time,"N/A");


		if(temp_status->status==SERVICE_PENDING){
			strcpy(status,"PENDING");
			strcpy(status_color,PENDING_COLOR);
		        }
		else if(temp_status->status==SERVICE_OK){
			strcpy(status,"OK");
			strcpy(status_color,OK_COLOR);
		        }
		else if(temp_status->status==SERVICE_RECOVERY){
			strcpy(status,"RECOVERY");
			strcpy(status_color,RECOVERY_COLOR);
		        }
		else if(temp_status->status==SERVICE_WARNING){
			strcpy(status,"WARNING");
			strcpy(status_color,WARNING_COLOR);
			strcpy(status_bg_color,WARNING_BG_COLOR);
		        }
		else if(temp_status->status==SERVICE_UNKNOWN){
			strcpy(status,"UNKNOWN");
			strcpy(status_color,UNKNOWN_COLOR);
			strcpy(status_bg_color,UNKNOWN_BG_COLOR);
		        }
		else if(temp_status->status==SERVICE_CRITICAL){
			strcpy(status,"CRITICAL");
			strcpy(status_color,CRITICAL_COLOR);
			strcpy(status_bg_color,CRITICAL_BG_COLOR);
		        }
		else if(temp_status->status==SERVICE_HOST_DOWN){
			strcpy(status,"HOST DOWN");
			strcpy(status_color,CRITICAL_COLOR);
			strcpy(status_bg_color,CRITICAL_BG_COLOR);
		        }
		else if(temp_status->status==SERVICE_UNREACHABLE){
			strcpy(status,"UNREACHABLE");
			strcpy(status_color,CRITICAL_COLOR);
			strcpy(status_bg_color,CRITICAL_BG_COLOR);
		        }


		if(odd)
			odd=0;
		else
			odd=1;
		if(temp_status->status==SERVICE_OK || temp_status->status==SERVICE_PENDING)
			printf("<TR bgcolor=%s>\n",(odd)?HTML_STATUS_BG_ODD_COLOR:HTML_STATUS_BG_EVEN_COLOR);
		else
			printf("<TR bgcolor=%s>\n",status_bg_color);
		printf("<TD><A HREF=\"%shosts.html#%s\"><FONT SIZE=-1>%s</FONT></A></TD>\n",url_html_path,temp_status->host_name,temp_status->host_name);
		printf("<TD><A HREF=\"%sservices.html#%s;%s\"><FONT SIZE=-1>%s</FONT></A></TD>\n",url_html_path,temp_status->host_name,temp_status->description,temp_status->description);	
		printf("<TD bgcolor=\"%s\"><A HREF=\"%sstatuslevels.html#%s\"><FONT SIZE=-1>%s</FONT></A></TD>\n",status_color,url_docs_path,status,status);
		printf("<TD><FONT SIZE=-1>%s</FONT></TD>\n",date_time);
		printf("<TD><FONT SIZE=-1>%d/%d</FONT></TD>\n",temp_status->current_attempt,temp_status->max_attempts);
		printf("<TD><FONT SIZE=-1>%s</FONT></TD>\n",temp_status->information);
		printf("</TR>\n");

	        }

	printf("</TABLE>\n");
	printf("</DIV>\n");
	printf("</P>\n");

	return;
        }



void show_hostgroup_overviews(void){
	hostgroup *temp_hostgroup=NULL;
	int current_column;


	/* display status overviews for all hostgroups */
	if(show_all_hostgroups==TRUE){

		printf("<P>\n");
		printf("<DIV ALIGN=CENTER>\n");
		printf("<font size=+1>Status Overview For All Host Groups</font>\n");
		printf("</DIV>\n");
		printf("</P>\n");

		printf("<DIV ALIGN=\"center\">\n");
		printf("<TABLE BORDER=0 CELLPADDING=10>\n");

		current_column=1;
		for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){

			if(current_column==1)
				printf("<TR>\n");
			printf("<TD VALIGN=\"top\" ALIGN=\"center\">\n");
				
			show_hostgroup_overview(temp_hostgroup);

			printf("</TD>\n");
			if(current_column==overview_columns)
				printf("</TR>\n");

			if(current_column<overview_columns)
				current_column++;
			else
				current_column=1;
		        }

		if(current_column!=1){

			for(;current_column<=overview_columns;current_column++)
				printf("<TD></TD>\n");
			printf("</TR>\n");
		        }

		printf("</TABLE>\n");
		printf("</DIV>\n");
	        }

	/* else display overview for just a specific hostgroup */
	else{

		temp_hostgroup=find_hostgroup(hostgroup_name);
		if(temp_hostgroup!=NULL){

			printf("<P>\n");
			printf("<DIV ALIGN=CENTER>\n");
			printf("<font size=+1>Status Overview For Host Group '%s'</font>\n",hostgroup_name);
			printf("</DIV>\n");
			printf("</P>\n");

			printf("<P>\n");
			printf("<DIV ALIGN=CENTER>\n");
			
			show_hostgroup_overview(temp_hostgroup);

			printf("</DIV>\n");
			printf("</P>\n");
		        }
		else
			printf("<center><font color=red><strong>Error: Host group '%s' doesn't seem to exist...</strong></font></center>",hostgroup_name);
	        }

	return;
        }



void show_hostgroup_overview(hostgroup *hstgrp){
	hoststatus *temp_hoststatus=NULL;
	int odd=0;

	printf("<table border=0>\n");
	printf("<tr>\n");
	printf("<td valign=center>\n");
	printf("<font size=-1><A HREF=\"status.cgi?hostgroup=%s&style=detail%s\">%s</A></font>",hstgrp->group_name,(display_popup==TRUE)?"":"&nopopup",hstgrp->group_alias);
	printf("</td>\n");
	printf("<td valign=center>\n");
	printf("<font size=-1>(<A HREF=\"%shostgroups.html#%s\">%s</A>)</font>",url_html_path,hstgrp->group_name,hstgrp->group_name);
	printf("</td>\n");
	printf("</table>\n");

	printf("<table border=1 bgcolor='#EEEEEE'>\n");

	printf("<tr bgcolor='%s'>\n",HTML_STATUS_BG_COLOR);
	printf("<td><font color='%s' size=-1><strong>Host Name</strong></font></td>\n",HTML_STATUS_FG_COLOR);
	printf("<td><font color='%s' size=-1><strong>Host Status</strong></font></td>\n",HTML_STATUS_FG_COLOR);
	printf("<td><font color='%s' size=-1><strong>Service Status Totals</strong></font></td>\n",HTML_STATUS_FG_COLOR);
	printf("<td><font color='%s' size=-1><strong>Actions</strong></font></td>\n",HTML_STATUS_FG_COLOR);
	printf("</tr>\n");

	/* find all the hosts that belong to the hostgroup */
	for(temp_hoststatus=hoststatus_list;temp_hoststatus!=NULL;temp_hoststatus=temp_hoststatus->next){

		/* is this host a member of the hostgroup */
		if(is_member_of_hostgroup(hstgrp,temp_hoststatus->host_name)){

			if(odd)
				odd=0;
			else
				odd=1;
			show_hostgroup_member_overview(temp_hoststatus,odd);
		        }
	        }

	printf("</table>\n");

	return;
        }

 

void show_hostgroup_member_overview(hoststatus *hststatus,int odd){
	char status[MAX_INPUT_BUFFER];
	char status_color[MAX_INPUT_BUFFER];


	if(hststatus->status==HOST_PENDING){
		strcpy(status,"PENDING");
		strcpy(status_color,PENDING_COLOR);
	        }
	else if(hststatus->status==HOST_UP){
		strcpy(status,"UP");
		strcpy(status_color,OK_COLOR);
	        }
	else if(hststatus->status==HOST_DOWN){
		strcpy(status,"DOWN");
		strcpy(status_color,CRITICAL_COLOR);
	        }
	else if(hststatus->status==HOST_UNREACHABLE){
		strcpy(status,"UNREACHABLE");
		strcpy(status_color,CRITICAL_COLOR);
	        }

	if(hststatus->status==HOST_UP || hststatus->status==HOST_PENDING)
		printf("<TR bgcolor=%s>\n",(odd)?HTML_STATUS_BG_ODD_COLOR:HTML_STATUS_BG_EVEN_COLOR);
	else
		printf("<TR bgcolor=%s>\n",CRITICAL_BG_COLOR);

	printf("<td><A HREF=\"%shosts.html#%s\"><font size=-2>%s</font></A></td>\n",url_html_path,hststatus->host_name,hststatus->host_name);

	printf("<td bgcolor=%s><A HREF=\"%sstatuslevels.html\"><font size=-2>%s</font></A></td>\n",status_color,url_docs_path,status);

	printf("<td>\n");
	show_hostgroup_member_service_status_totals(hststatus->host_name);
	printf("</td>\n");

	printf("<td><a href=\"status.cgi?host=%s%s\"><img src=\"%s%s\" border=0 alt=\"View Service Details For This Host\"></a></td>\n",hststatus->host_name,(display_popup==TRUE)?"":"&nopopup",url_images_path,STATUS_DETAIL_ICON);

	printf("</TR>\n");

	return;
        }



void show_hostgroup_member_service_status_totals(char *host_name){
	int total_ok=0;
	int total_warning=0;
	int total_unknown=0;
	int total_critical=0;
	int total_pending=0;
	servicestatus *temp_servicestatus;


	for(temp_servicestatus=servicestatus_list;temp_servicestatus!=NULL;temp_servicestatus=temp_servicestatus->next){

		if(!strcmp(host_name,temp_servicestatus->host_name)){

			if(temp_servicestatus->status==SERVICE_CRITICAL)
				total_critical++;
			else if(temp_servicestatus->status==SERVICE_HOST_DOWN)
				total_critical++;
			else if(temp_servicestatus->status==SERVICE_UNREACHABLE)
				total_critical++;
			else if(temp_servicestatus->status==SERVICE_WARNING)
				total_warning++;
			else if(temp_servicestatus->status==SERVICE_UNKNOWN)
				total_unknown++;
			else if(temp_servicestatus->status==SERVICE_RECOVERY)
				total_ok++;
			else if(temp_servicestatus->status==SERVICE_OK)
				total_ok++;
			else if(temp_servicestatus->status==SERVICE_PENDING)
				total_pending++;
			else
				total_ok++;
		        }
	        }


	printf("<TABLE BORDER=\"0\" bgcolor=\"#EEEEEE\">\n");

	printf("<TR>\n");
	if(total_ok>0){
		printf("<TD align=\"center\"");
	        printf(" bgcolor=\"%s\"",OK_COLOR);
		printf("><font size=-2>%d <A HREF=\"%sstatuslevels.html\">OK</A></font></TD>\n",total_ok,url_docs_path);
	        }
	if(total_warning>0){
		printf("<TD align=\"center\"");
	        printf(" bgcolor=\"%s\"",WARNING_COLOR);
		printf("><font size=-2>%d <A HREF=\"%sstatuslevels.html\">WARNING</A></font></TD>\n",total_warning,url_docs_path);
	        }
	if(total_unknown>0){
		printf("<TD align=\"center\"");
	        printf(" bgcolor=\"%s\"",UNKNOWN_COLOR);
		printf("><font size=-2>%d <A HREF=\"%sstatuslevels.html\">UNKNOWN</A></font></TD>\n",total_unknown,url_docs_path);
	        }
	if(total_critical>0){
		printf("<TD align=\"center\"");
	        printf(" bgcolor=\"%s\"",CRITICAL_COLOR);
		printf("><font size=-2>%d <A HREF=\"%sstatuslevels.html\">CRITICAL</A></font></TD>\n",total_critical,url_docs_path);
	        }
	if(total_pending>0){
		printf("<TD align=\"center\"");
	        printf(" bgcolor=\"%s\"",PENDING_COLOR);
		printf("><font size=-2>%d <A HREF=\"%sstatuslevels.html\">PENDING</A></font></TD>\n",total_pending,url_docs_path);
	        }

	printf("</TR>\n");
	printf("</TABLE>\n");

	return;
        }
